import { recursivelyObservable } from "./observable.js";
import { SqliteState } from "./sqlite.js";
import { unreachable } from "./util.js";
export * from "./decorator.js";
export function state(ctx, name, initialState, options = {
    snapshotPolicy: { changes: 10 },
}) {
    const state = new SqliteState(name, ctx.storage);
    const data = state.resume(initialState);
    return recursivelyObservable(data, {
        onUpdate(changes, data) {
            state.appendChanges(changes);
            maybeSnapshot(data, state, options.snapshotPolicy ?? { changes: 10 });
        },
    });
}
function maybeSnapshot(data, state, snapshotPolicy) {
    if (snapshotPolicy === "every-change") {
        state.snapshot(data);
    }
    else if (typeof snapshotPolicy === "object" &&
        "changes" in snapshotPolicy) {
        const latest = state.latestChange() ?? unreachable();
        if (latest.id % snapshotPolicy.changes === 0) {
            state.snapshot(data);
        }
    }
}
